TTF_FontStyleFlags

Type Alias TTF_FontStyleFlags 

pub type TTF_FontStyleFlags = u32;
Font style flags for TTF_Font

These are the flags which can be used to set the style of a font in SDL_ttf. A combination of these flags can be used with functions that set or query font style, such as TTF_SetFontStyle or TTF_GetFontStyle.

Availability

This function is available since SDL_ttf 3.0.0.

See also

Known values (sdl3-sys)

ConstantDescription
TTF_STYLE_NORMALNo special style
TTF_STYLE_BOLDBold style
TTF_STYLE_ITALICItalic style
TTF_STYLE_UNDERLINEUnderlined text
TTF_STYLE_STRIKETHROUGHStrikethrough text