Display

Struct Display 

pub struct Display { /* private fields */ }

Implementations

Source

impl Display

pub fn is_connected(&self) -> bool

pub fn get_properties(&self) -> Result<Properties, PropertiesError>

pub fn get_name(&self) -> Result<String, Error>

Get the name of the display at the index display_name.

Will return an error if the index is out of bounds or if SDL experienced a failure; inspect the returned string for further info.

pub fn get_bounds(&self) -> Result<Rect, Error>

pub fn get_usable_bounds(&self) -> Result<Rect, Error>

pub fn get_fullscreen_modes(&self) -> Result<Vec<DisplayMode>, Error>

pub fn get_mode(&self) -> Result<DisplayMode, Error>

pub fn get_closest_display_mode( &self, mode: &DisplayMode, include_high_density_modes: bool, ) -> Result<DisplayMode, Error>

pub fn get_orientation(&self) -> SDL_DisplayOrientation

Return orientation of a display or Unknown if orientation could not be determined.

pub fn get_natural_orientation(&self) -> SDL_DisplayOrientation

Return orientation of a display or Unknown if orientation could not be determined.

pub fn get_content_scale(&self) -> Result<f32, Error>