pub unsafe extern "C" fn TTF_CreateRendererTextEngineWithProperties(
props: u32,
) -> *mut TTF_TextEngineExpand description
Create a text engine for drawing text on an SDL renderer, with the specified properties.
These are the supported properties:
TTF_PROP_RENDERER_TEXT_ENGINE_RENDERER: the renderer to use for creating textures and drawing textTTF_PROP_RENDERER_TEXT_ENGINE_ATLAS_TEXTURE_SIZE: the size of the texture atlas
Parameters
props: the properties to use.
Return value
Returns a TTF_TextEngine object or NULL on failure; call SDL_GetError()
for more information.
Thread safety
This function should be called on the thread that created the renderer.
Availability
This function is available since SDL_ttf 3.0.0.