pub type TTF_FontStyleFlags = u32;Expand description
Font style flags for TTF_Font
These are the flags which can be used to set the style of a font in
SDL_ttf. A combination of these flags can be used with functions that set
or query font style, such as TTF_SetFontStyle or TTF_GetFontStyle.
Availability
This function is available since SDL_ttf 3.0.0.
See also
Known values (sdl3-sys)
| Constant | Description |
|---|---|
TTF_STYLE_NORMAL | No special style |
TTF_STYLE_BOLD | Bold style |
TTF_STYLE_ITALIC | Italic style |
TTF_STYLE_UNDERLINE | Underlined text |
TTF_STYLE_STRIKETHROUGH | Strikethrough text |